@@ -127,6 +127,8 @@ build_config(char *prefix, struct manager_ctx *manager, struct server *server)
         fprintf(f, ",\n\"no_delay\": %s", server->no_delay);
     else if (manager->no_delay)
         fprintf(f, ",\n\"no_delay\": true");
+    if (manager->reuse_port)
+        fprintf(f, ",\n\"reuse_port\": true");
     if (server->mode)
         fprintf(f, ",\n\"mode\":\"%s\"", server->mode);
     if (server->plugin)
@@ -1052,6 +1054,9 @@ main(int argc, char **argv)
         if (acl == NULL) {
             acl = conf->acl;
         }
+        if (manager_address == NULL) {
+            manager_address = conf->manager_address;
+        }
 #ifdef HAVE_SETRLIMIT
         if (nofile == 0) {
             nofile = conf->nofile;
