@@ -360,6 +360,10 @@ usage()
 #endif
     printf(
         "       [-U]                       Enable UDP relay and disable TCP relay.\n");
+#ifdef MODULE_REDIR
+    printf(
+        "       [-T]                       Use tproxy instead of redirect (for tcp).\n");
+#endif
 #ifdef MODULE_REMOTE
     printf(
         "       [-6]                       Resovle hostname to IPv6 address first.\n");
@@ -546,6 +550,7 @@ get_default_conf(void)
         return userconf;
 
     // If not, fall back to the system-wide config.
+    free(userconf);
     return sysconf;
 #else
     return "config.json";
